%PALIJE

% Izrauna sile in reakcije podpor ravninskega palija podanega s podatki voz,pq,Em,A,podpore,obt.
% voz..koordinate vozli,
% pq..palice, 
% E.. modul elastinosti
% A.. ploina preseka
% podpore ..vektor fiksnih pomikov
% obt.. nenielne komponente zunanjih sil. 
%
% Dejan Zupan, Katedra za mehaniko, .l. 2003/04

%podatki

% ********************************************************
% * BRANJE VHODNIH PODATKOV IN NJIHOVO PRIREJANJE RAUNU *
% ********************************************************

%podatki

%tevilo vozli konstrukcije
st_voz=size(voz,1);

%tevilo elementov konstrukcije
st_el=size(pq,1);

% Vektor elastinih modulov palic  
if prod(size(Em))==1
   El=Em*ones(st_el,1); %vse palice imajo enak elastini modul
else
   El=Em(1,2)*ones(st_el,1); %za vse palice,ki niso podane se vzame prva vrednost
   El(Em(:,1))=Em(:,2);   %tiste, ki so podane se vpie na prava mesta
end

% Vektor ploin prenih prerezov palic  
if prod(size(A))==1
   Al=A*ones(st_el,1); %vse palice imajo enako ploino
else
   Al=A(1,2)*ones(st_el,1); %za vse,ki niso podane se vzame prva vrednost
   Al(A(:,1))=A(:,2);   %tiste, ki so podane se vpie na prava mesta
end

%podprte prostostne stopnje
fiks=[];
for i=1:size(podpore,1)
   fiks=union(fiks,nonzeros(podpore(i,2:size(podpore,2)))+2*(podpore(i,1)-1));
end
prost=1:2*st_voz;
prost(fiks)=[];  %index pomeni vse proste prostostne stopnje

% obteni vektor  
f_obt=zeros(2*st_voz,1);
for i=1:size(obt,1)
   f_obt(2*obt(i,1)-1:2*obt(i,1))=obt(i,2:3);
end

% *****************************************
% * RAUNU DOLIN PALIC IN OSNIH TOGOSTI *
% *****************************************

% Vektor dolin palic
for i=1:st_el
  L(i,1)=((voz(pq(i,2),2)-voz(pq(i,3),2))^2+(voz(pq(i,2),3)-voz(pq(i,3),3))^2)^0.5;
end

% Matrika smernih kosinusov palija
for j=1:size(pq,1)
  Sc(j,2*pq(j,2)-1)=(voz(pq(j,3),2)-voz(pq(j,2),2))/L(j);
  Sc(j,2*pq(j,2))=(voz(pq(j,3),3)-voz(pq(j,2),3))/L(j);
  Sc(j,2*pq(j,3)-1)=-Sc(j,2*pq(j,2)-1);
  Sc(j,2*pq(j,3))=-Sc(j,2*pq(j,2));
end

% Diagonalna osnih togosti k
for j=1:st_el
  k(j,j)=El(j)*Al(j)/L(j); 
end 


% **********************************
% * IZPIS TABELE POMEMBNIH KOLIIN *
% **********************************

disp(sprintf('\n\n TABELA DOLIN, KOSINUSOV IN OSNIH TOGOSTI ZA PODANO PALIJE'))
disp(' ====================================================================================')
disp(sprintf('%8s %8s %8s %12s %12s %12s %8s','palica','vozel1', 'vozel2','dolzina', 'cos(a_ij)','cos(b_ij)','k_ij' ))
disp(' ====================================================================================')
for i=1:st_el
   disp(sprintf('%6i %8i %8i %12.3f %12.3f %12.3f %12.3f ' ,...\n
      				i,voz(pq(i,2),1),voz(pq(i,3),1),L(i),Sc(i,2*pq(i,2)-1),Sc(i,2*pq(i,2)),k(i,i) ))
	disp(' ---------------------------------------------------------------------------------')
end

% *************************************
% * IZRAUN IN IZPIS TOGOSTNE MATRIKE *
% *************************************

% Togostna matrika palija
K=-Sc'*k*Sc;


disp(sprintf('\n\n TOGOSTNA MATRIKA PALIJA'))
disp(' ====================================================================================')
for i=1:2*st_voz
   disp(sprintf('%10.3f ' ,K(i,:) ))
end

% ****************************
% * RAUN POMIKOV IN REAKCIJ *
% ****************************

%fiks
% Odstranitev podprtih prostostnih stopenj
K_tilda=K;
K_tilda(fiks,:)=[];
K_tilda(:,fiks)=[];
f_obt_tilda=f_obt;
f_obt_tilda(fiks)=[];

% Izraunamo vektor nenielnih pomikov u
u_tilda=K_tilda\(-f_obt_tilda);

% Razpihnemo vektor pomikov u z nilami v celoten vektor pomikov
u=zeros(2*st_voz,1);
u(prost)=u_tilda;


% Izraunamo reakcije podpor
R=-K*u-f_obt;
Reakcije=R(fiks);

% ********************
% * IZPIS REZULTATOV *
% ********************

disp(sprintf('\n\n POMIKI IN REAKCIJE VOZLI DANEGA PALIJA'))
disp(' ==================================================================================')
disp(sprintf('%8s %12s %15s %15s %15s','vozel', 'u_x','u_y', 'R_x','R_y' ))
disp(' ==================================================================================')
for i=1:st_voz
   if any(2*i-1==fiks)& any(2*i==fiks)
   	disp(sprintf('%6i %15.5f %15.5f %15.3f %15.3f ' ,i,u(2*i-1,1),u(2*i,1),R(2*i-1),R(2*i) ))
		disp(' --------------------------------------------------------------------------------')
   elseif any(2*i-1==fiks)
   	disp(sprintf('%6i %15.5f %15.5f %15.3f %15.3f ' ,i,u(2*i-1,1),u(2*i,1),R(2*i-1) ))
		disp(' --------------------------------------------------------------------------------')
   elseif any(2*i==fiks)
   	disp(sprintf('%6i %15.5f %15.5f %15s %15.3f ' ,i,u(2*i-1,1),u(2*i,1),'' ,R(2*i) ))
		disp(' --------------------------------------------------------------------------------')
   else
   	disp(sprintf('%6i %15.5f %15.5f %15.3f %15.3f ' ,i,u(2*i-1,1),u(2*i,1) ))
		disp(' --------------------------------------------------------------------------------')
   end
end


% ****************************
% * RAUN OSNIH SIL PALIJA  *
% ****************************

% Vektor dolin palic
for i=1:st_el
  N(i)=k(i,i)*[Sc(i,2*pq(i,2)-1) Sc(i,2*pq(i,2))]*[ ( u(2*pq(i,3)-1) - u(2*pq(i,2)-1));   (u(2*pq(i,3))   - u(2*pq(i,2)) )  ];
end

% ********************************
% * IZPIS TABELE NOTRANJIH SIL  *
% ********************************

disp(sprintf('\n\n TABELA OSNIH SIL ZA PODANO PALIJE'))
disp(' ==================================================')
disp(sprintf('%8s %8s %8s %8s','palica','vozel1', 'vozel2','N_ij' ))
disp(' ==================================================')
for i=1:st_el
   disp(sprintf('%6i %8i %8i %12.3f ' ,...\n
      				i,voz(pq(i,2),1),voz(pq(i,3),1),N(i) ))
	disp(' -----------------------------------------------')
end


% **************************
% * RISANJE GRAFA PALIJA  *
% **************************

% nariemo palije zunanjo obtebo in reakcije
graf(voz,pq,podpore,obt);

beri=input('\n  Poveava risanja pomikov (10 ): ');
if isempty(beri)
	faktor=10;
else
  	faktor=beri;
end

    
prirastki=faktor*reshape(u,2,size(u,1)/2);
Td=voz(:,2:3)+prirastki';

%risanje deformiranih palic
for i=1:size(pq,1)
  t=plot(Td(pq(i,2:3),1),Td(pq(i,2:3),2),'g'); %riemo povezave iz pq
  set(t, 'LineWidth', 1.2); 
end

figure(1)
